% TEST_UIUCCLEANER Unitary tests for the uiuccleaner function
% Note:
%   Matlab package functionality is not very test-framework friendly.
%   Either the whole package has to be imported in EVERY SINGLE TEST or the
%   functions must be called as <package>.function() everytime.
%   The second option is preferred for the tests as it has the smaller scope.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Main test function

function tests = test_uiuccleaner

tests = functiontests(localfunctions);

end


%% Setup and teardown

function setupOnce(testCase)

addpath('../.'); % Add repository to Matlab Path
addpath('./test_utils'); % Add utils to Matlab Path

% Set random number generator settings.
testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)

rmpath('../.'); % Remove repository from Matlab Path
rmpath('./test_utils'); % Remove utils from Matlab Path

% Restore the random number generator settings.
s = testCase.TestData.currentRNG;
rng(s)

end

function teardown(~)

close all; % Close all figures that would have been openend

end


%% Arguments and errors tests

function test_nargin(testCase)
% Error if too many arguments

dummy = (1:10)';

verifyError(testCase, @() af_tools.uiuccleaner(dummy, dummy, dummy, 'autosave',true,'overwrite',false), 'MATLAB:narginchk:tooManyInputs')

end


function test_invalidInputsType(testCase)
% Error if base inputs are of the wrong type

empty = [];
scal = 0;
logi = true;
Struct = struct;

% 1. inputDir (char vector or string)
verifyError(testCase, @() af_tools.uiuccleaner(empty), 'MATLAB:parsefileinputs:invalidType')
verifyError(testCase, @() af_tools.uiuccleaner(scal), 'MATLAB:parsefileinputs:invalidType')
verifyError(testCase, @() af_tools.uiuccleaner(logi), 'MATLAB:parsefileinputs:invalidType')
verifyError(testCase, @() af_tools.uiuccleaner(Struct), 'MATLAB:parsefileinputs:invalidType')

% 2. inputFile (char, string, cell of chars)
verifyError(testCase, @() af_tools.uiuccleaner(scal, empty), 'MATLAB:parsefileinputs:invalidType')
verifyError(testCase, @() af_tools.uiuccleaner(scal, scal), 'MATLAB:parsefileinputs:invalidType')
verifyError(testCase, @() af_tools.uiuccleaner(scal, logi), 'MATLAB:parsefileinputs:invalidType')
verifyError(testCase, @() af_tools.uiuccleaner(scal, Struct), 'MATLAB:parsefileinputs:invalidType')

end


function test_invalidOptions(testCase)
% Error if invalid option name or missing parameter value

wrongName = 'wrongOption';
testdir = [pwd,'/test_utils'];

verifyError(testCase, @() af_tools.uiuccleaner(testdir, '*', wrongName), 'MATLAB:InputParser:ParamMissingValue')
verifyError(testCase, @() af_tools.uiuccleaner(testdir, '*', wrongName, true), 'MATLAB:InputParser:UnmatchedParameter')
verifyError(testCase, @() af_tools.uiuccleaner(testdir, 'autosave', true, wrongName), 'MATLAB:InputParser:ParamMissingValue')
verifyError(testCase, @() af_tools.uiuccleaner(testdir, 'autosave', true, wrongName, true), 'MATLAB:InputParser:UnmatchedParameter')

end


function test_invalidOptionsVal(testCase)
% Error if valid option name but invalid value

empty = [];
scal = 0;
char = 'test';
Struct = struct;
vect = 1:10;

testdir = [pwd,'/test_utils'];

verifyError(testCase, @() af_tools.uiuccleaner(testdir, 'autosave', empty), 'MATLAB:uiuccleaner:invalidType')
verifyError(testCase, @() af_tools.uiuccleaner(testdir, 'autosave', scal), 'MATLAB:uiuccleaner:invalidType')
verifyError(testCase, @() af_tools.uiuccleaner(testdir, 'autosave', char), 'MATLAB:uiuccleaner:invalidType')
verifyError(testCase, @() af_tools.uiuccleaner(testdir, 'autosave', Struct), 'MATLAB:uiuccleaner:invalidType')
verifyError(testCase, @() af_tools.uiuccleaner(testdir, 'autosave', vect), 'MATLAB:uiuccleaner:invalidType')

verifyError(testCase, @() af_tools.uiuccleaner(testdir, 'overwrite', empty), 'MATLAB:uiuccleaner:invalidType')
verifyError(testCase, @() af_tools.uiuccleaner(testdir, 'overwrite', scal), 'MATLAB:uiuccleaner:invalidType')
verifyError(testCase, @() af_tools.uiuccleaner(testdir, 'overwrite', char), 'MATLAB:uiuccleaner:invalidType')
verifyError(testCase, @() af_tools.uiuccleaner(testdir, 'overwrite', Struct), 'MATLAB:uiuccleaner:invalidType')
verifyError(testCase, @() af_tools.uiuccleaner(testdir, 'overwrite', vect), 'MATLAB:uiuccleaner:invalidType')

end


% -------------------------------------
function test_dirNotFound(testCase)
% Error if valid inputDir not found

verifyError(testCase, @() af_tools.uiuccleaner('missingdir'), 'MATLAB:parsefileinputs:dirNotFound')

end


function test_fileNotFound(testCase)
% Error if valid inputDir not found

testdir = [pwd,'/test_utils'];

verifyWarning(testCase, @() af_tools.uiuccleaner(testdir, {'clarky.dat','testdummy.dat'}), 'MATLAB:parsefileinputs:FileNotFound')
warning('off','MATLAB:parsefileinputs:FileNotFound') % remove warning first warning about FileNotFound before it throws the error
verifyError(testCase, @() af_tools.uiuccleaner(testdir, 'randomFile'), 'MATLAB:parsefileinputs:noFilesFound')
warning on % Reactivate warnings

end



%% Test if function returns expected outputs

function test_correctOrder(testCase)
% Verify if the output starts and ends at X = 1;

testdir = [pwd,'/test_utils'];

Dat = af_tools.uiuccleaner(testdir, '*');

verifyEqual(testCase, Dat(1).x(1), 1)
verifyEqual(testCase, Dat(1).x(end), 1)
verifyEqual(testCase, Dat(2).x(end), 1)
verifyEqual(testCase, Dat(2).x(end), 1)

end
